package com.webage.ejbs.bank.account;

import java.util.List;

import javax.annotation.security.DeclareRoles;
import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

//@DeclareRoles( { "admin", "user" })
//@RolesAllowed("admin")
@Stateless(name = "accountManager", mappedName = "ejb/accountManagerJNDI")
@PermitAll
public class AccountManager implements AccountManagerLocal,
		AccountManagerRemote {
	@PersistenceContext(unitName = "accounts")
	EntityManager em;

	public static final String RemoteJNDIName = "ejb/accountManagerJNDI";

	@Override
	//@RolesAllowed("user")
	public double getAccountBalance(int accountNb) {
		return em.find(Account.class, accountNb).balance;
	}

	@Override
	//@RolesAllowed("user")
	public Account getAccount(int accountNb) {
		return em.find(Account.class, accountNb);
	}

	@Override
	//@RolesAllowed("user")
	public int createAccount(double balance) throws AccountException {
		Account a = new Account(balance);
		em.persist(a);
		return a.accountNb;
	}

	@Override
	//@RolesAllowed("user")
	public void closeAccount(int accountNb) {
		em.remove(em.find(Account.class, accountNb));
	}

	@Override
	//@RolesAllowed("user")
	public List<Account> findAccountLargerThan(double balance) {
		Query q = em
				.createQuery("select a from Account a where a.balance > :pbalance order by a.balance");
		q.setParameter("pbalance", balance);
		List<Account> result = (List<Account>) q.getResultList();
		return result;
	}

	@Override
	//@RolesAllowed("user")
	public void updateAccount(Account account) {
		em.merge(account);
	}

	@Override
	//@RolesAllowed("user")
	public void credit(int accountNb, double amount) {
		Account a;
		a = em.find(Account.class, accountNb);
		a.credit(amount);
		updateAccount(a);
	}

	@Override
	//@RolesAllowed("user")
	public void debit(int accountNb, double amount) {
		Account a;
		a = em.find(Account.class, accountNb);
		a.debit(amount);
		updateAccount(a);
	}

	@Override
	//@RolesAllowed("admin")
	public String restricted() {
		return "Ok pour méthode restreinte à admin";
	}

}